<?php

namespace Service;

use Factory\CustomizationFilesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Traits\TResponse;
use Service\BilletIssued;
use Util\InstanceProvider;


class CustomizationFilesService
{
    use TResponse;

    /** @var \Teknisa\Model\Teknisa\Repositories\Licencauso $licencausoRepository */
    protected  $licencausoRepository;

    /** @var \Teknisa\Model\Teknisa\Repositories\Linpluso $useLicenseRepository */
    protected  $useLicenseRepository;

    protected $CustomizationFilesFactory;
    protected $EntityManager;
    protected $Environment;
    protected $newCode;
    protected $BilletIssued;

    public function __construct(CustomizationFilesFactory $CustomizationFilesFactory, Environment $Environment, NewCode $newCode, EntityManager $EntityManager, BilletIssued $BilletIssued)
    {
        $this->licencausoRepository = InstanceProvider::getRepository()->getLicencauso();
        $this->useLicenseRepository = InstanceProvider::getRepository()->getLinpuso();
        $this->EntityManager = $EntityManager;
        $this->CustomizationFilesFactory    = $CustomizationFilesFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->BilletIssued                 = $BilletIssued;
    }

    public function getPais()
    {
        return $this->CustomizationFilesFactory->getPais();
    }

    public function getEstado($cdpais)
    {
        return $this->CustomizationFilesFactory->getEstado($cdpais);
    }

    public function getMunicipio($sgestado)
    {
        return $this->CustomizationFilesFactory->getMunicipio($sgestado);
    }

    public function getUseLicense()
    {
        $cdCliente = Utilities::getEnvironment()->getCurrentCdCliente();
        return $this->CustomizationFilesFactory->getUseLicense($cdCliente);
    }

    public function downloadPlanilhaPadrao()
    {
        $url = Utilities::getPlanilhaPadraoFilePersonalization();
        $nomeArquivo = 'Planilha_Padrao.xlsx';
        $tipo = 'xlsx';
        Utilities::downloadPlanilhaPadrao($url, $nomeArquivo, $tipo);
    }

    public function downloadPlanilhaPadraoInPER()
    {
        $url = Utilities::getPlanilhaPadraoFilePersonalizationInPER();
        $nomeArquivo = 'Planilha_Padrao_Inativar_PER.xlsx';
        $tipo = 'xlsx';
        Utilities::downloadPlanilhaPadrao($url, $nomeArquivo, $tipo);
    }


    public function validaArquivoPersonalizado($file)
    {
        // A string codificada fornecida
        $encoded_data = $file[0]['b64File'];

        $data_start = strpos($encoded_data, ',') + 1;
        $encoded_data = substr($encoded_data, $data_start);
        $decoded_data = base64_decode($encoded_data);

        // Salve os dados em um arquivo temporário (opcional)
        $temp_file = tempnam(sys_get_temp_dir(), 'excel_');
        file_put_contents($temp_file, $decoded_data);

        $reader = \PHPExcel_IOFactory::createReaderForFile($temp_file);
        $spreadsheet = $reader->load($temp_file);

        // Obtenha a primeira planilha (você pode ajustar isso se houver várias planilhas)
        $worksheet = $spreadsheet->getActiveSheet();

        // Inicialize um array para armazenar os dados das linhas
        $data = [];

        // Itere pelas linhas com dados
        foreach ($worksheet->getRowIterator() as $row) {
            // Obtenha todas as células da linha
            $cellIterator = $row->getCellIterator();
            $cellIterator->setIterateOnlyExistingCells(false); // Inclui células vazias
            $rowData = [];

            // Itere pelas células da linha
            foreach ($cellIterator as $cell) {
                $rowData[] = $cell->getValue();
            }

            // Adicione os dados da linha ao array de dados
            $data[] = $rowData;
        }

        // Agora você tem todos os dados da planilha em formato de matriz

        // Inicialize um array para armazenar os dados válidos (sem linhas vazias)
        $validData = [];

        foreach ($data as $rowData) {
            // Verifique se pelo menos uma célula na linha possui um valor não vazio
            if (array_filter($rowData)) {
                // Adicione apenas linhas não vazias ao array de dados válidos
                $validData[] = $rowData;
            }
        }

        unlink($temp_file);

        // Agora, $validData contém apenas as linhas não vazias da planilha
        return $validData;
    }

    public function validateCDLOCALUSO($dataArray) {
        $uniqueCDLOCALUSOSet = [];
    
        foreach ($dataArray as $item) {
            if (in_array($item["CDLOCALUSO"], $uniqueCDLOCALUSOSet)) {
                return false;
            }
    
            $uniqueCDLOCALUSOSet[] = $item["CDLOCALUSO"];
        }
    
        return true;
    }

    public function validaCdPais ($cdpais) {
        $arrayPais = $this->getPais();
        foreach ($arrayPais['data'] as $item) {
            if ($cdpais == $item['CDPAIS']) {
                return true;
            }
        }
        return false;
    }

    public function validaSgestado ($cdpais, $sgestado) {
        $arrayEstado = $this->getEstado($cdpais);
        foreach ($arrayEstado['data'] as $item) {
            if ($sgestado == $item['SGESTADO']) {
                return true;
            }
        }
        return false;
    }

    public function validaCdmunicipio ($sgestado, $cdmunicipio) {
        $arrayMunicipio = $this->getMunicipio($sgestado);
        foreach ($arrayMunicipio['data'] as $item) {
            if ($cdmunicipio == $item['CDMUNICIPIO']) {
                return true;
            }
        }
        return false;
    }
    
    public function saveCustomizationFiles($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                                            $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                                            $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                                            $cdlinpclie, $nrlicencauso, $idmodproc, $idlocaproc, $idtpijurluso, $rows) {
        $connection = $this->EntityManager->getConnection();
        $cdcliente = Utilities::getEnvironment()->getCurrentCdCliente();
        try {
            $cdlinpclie = explode(';', $cdlinpclie);
            $this->EntityManager->beginTransaction();
            /** @var  \Teknisa\Model\Teknisa\Entities\Licencauso $customizationFile*/
            $customizationFile = $this->licencausoRepository->findOneBy(array('cdcliente' => $cdcliente, 'cdlocaluso' => $cdlocaluso));
            $validateCDLOCALUSO = $this->validateCDLOCALUSO($rows);
            $validateCDPAIS = $this->validaCdPais($cdpais);
            $validateSGESTADO = true;
            $validateCDMUNICIPIO = true;
            if ($validateCDPAIS) {
                $validateSGESTADO = $this->validaSgestado($cdpais, $sgestado);
                if ($validateSGESTADO) {
                    $validateCDMUNICIPIO = $this->validaCdmunicipio($sgestado, $cdmunicipio);
                }
            }
            if (!$validateCDLOCALUSO) {
                throw new \Exception('Inclusão bloqueada, o Campo Local de Uso não pode estar duplicado no arquivo');
            } else if (!$validateCDPAIS) {
                throw new \Exception("Inclusão bloqueada, o código '$cdpais' no Campo País não existe!");
            } else if (!$validateSGESTADO) {
                throw new \Exception("Inclusão bloqueada, o código '$sgestado' no Campo UF não existe no País informado!");
            } else if (!$validateCDMUNICIPIO) {
                throw new \Exception("Inclusão bloqueada, o código '$cdmunicipio' no Campo Município não existe na UF informada!");
            } else if (!$customizationFile) {
                $customizationFile = $this->CustomizationFilesFactory->saveLicencauso($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                    $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                    $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                    $cdcliente, $nrlicencauso, $idmodproc, $idlocaproc, false, 'S', 1, $idtpijurluso);
            } else {
                $idAtivo = $customizationFile->getIdlusoativ() == 'S' ? 'Ativo' : 'Inativo';
                throw new \Exception('Inclusão bloqueada! Existe uma Licença de Uso já cadastrada com o código '.$cdlocaluso.'<br/>('.$customizationFile->getNmfantluso().' - '.$idAtivo.')');
            }
            $productLineRelatedToUseLicenses = $this->useLicenseRepository->findBy(array('cdcliente' => $cdcliente, 'nrlicencauso' => $nrlicencauso));
            $productLineRelatedToUseLicenseIdArray = array();
            foreach($productLineRelatedToUseLicenses as $productLineRelatedToUseLicense){
                if(!in_array($productLineRelatedToUseLicense->getCdlinpclie(), $cdlinpclie)){
                    $this->EntityManager->remove($productLineRelatedToUseLicense);
                }else{
                    array_push($productLineRelatedToUseLicenseIdArray, $productLineRelatedToUseLicense->getCdlinpclie());
                }
            }
            if($cdlinpclie){
                foreach($cdlinpclie as $userLicenseId) {
                    if (!in_array($userLicenseId, $productLineRelatedToUseLicenseIdArray)) {
                        $linpluso = $this->CustomizationFilesFactory->createLinpluso($cdcliente, $nrlicencauso, $userLicenseId);
                        $this->EntityManager->persist($linpluso);
                    }
                }
            }


            $this->EntityManager->persist($customizationFile);
            $this->EntityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function inativarLocaisUsoArquivoPersonalizado($arrayId) {
        $connection = $this->EntityManager->getConnection();
        $cdCliente  = Utilities::getEnvironment()->getCurrentCdCliente();
        try {
            $connection->beginTransaction();
            $arrayDividido = array_chunk($arrayId, 500);
            foreach ($arrayDividido as $arrayLicensauso) {
                $this->CustomizationFilesFactory->inativarLicensaUso($cdCliente, $arrayLicensauso);
            }
            /* $customizationFile = $this->licencausoRepository->findOneBy(array('cdcliente' => $cdCliente, 'nrlicencauso' => $licencauso));
            $customizationFile->setIdlusoativ('N');
            $this->EntityManager->flush(); */
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}
