import { Http } from '@zeedhi/core';
import { Modal, ModalService } from '@zeedhi/common';

export class NotificationController {
	private modalNotificationList!: Modal;

	private iFrameDocItems : any = {};

	private async getNotifications() {
        const uri = '/getNotifications';
        const request = await Http.get(uri);
        return request.data.dataset.getNotifications;
    }

    public async setIFrameDocSource({ component }: any) {
        const itemUrl = this.iFrameDocItems[component.name];
        if (itemUrl) {
            const doomIframe = document.getElementById(component.name) as HTMLIFrameElement;
            doomIframe.src = `${itemUrl}#zoom=110`;
            this.notificationOnMount({ component });
        }
    }

    public async notificationOnMount({ component }: any) {
        this.notificationMarkAsRead(component.parent.getChildInstance(`${component.parent.name}_id`).text);
    }

    public async notificationOnMountImg({ component }: any) {
        this.notificationMarkAsRead(component.parent.parent.getChildInstance(`${component.parent.parent.name}_id`).text);
    }

    public async closeNotificationsModal() {
        this.modalNotificationList.hide();
    }

    public async openNotifications() {
        const values = await this.getNotifications();
        if (values.length > 0) {
            const items = [];
            this.iFrameDocItems = {};
            this.modalNotificationList = await ModalService.createFromJson('dashboard/notificationList', true);
            for (let i = 0; i < values.length; i += 1) {
                const value = values[i];
                const name = `notification_${i}`;
                let contentContainer = {};
                if (value.IDNOTIFICACAO === 'P') {
                    const docId = `${name}_iframeDoc`;
                    contentContainer = {
                        name: docId,
                        id: docId,
                        component: 'ZdTag',
                        tag: 'iframe',
                        cssStyle: 'width: 100%; height: calc(100vh - 132px)',
                        events: {
                            onMounted: '{{NotificationController.setIFrameDocSource}}',
                        },
                    };
                    this.iFrameDocItems[docId] = value.TXCONTEUDOINF;
                } else if (value.IDNOTIFICACAO === 'I') {
                    contentContainer = {
                        name: `${name}_img_row`,
                        component: 'ZdRow',
                        justify: 'center',
                        align: 'center',
                        children: [
                            {
                                name: `${name}_content`,
                                component: 'ZdImage',
                                src: value.TXCONTEUDOINF,
                                events: {
                                    onMounted: '{{NotificationController.notificationOnMountImg}}',
                                },
                            },
                        ],
                    };
                } else {
                    contentContainer = {
                        name: `${name}_content`,
                        component: 'ZdText',
                        text: value.TXCONTEUDOINF,
                        events: {
                            onMounted: '{{NotificationController.notificationOnMount}}',
                        },
                    };
                }
                const item = {
                  component: 'ZdCollapseCard',
                  name,
                  lazyLoad: true,
                  expanded: i === 0,
                  class: 'zd-my-3',
                  color: 'grey lighten-5',
                  height: '10px',
                  expandButton: {
                    name: `${name}_icon`,
                    component: 'ZdButton',
                    icon: true,
                    iconName: 'dropdown',
                    color: 'secondary',
                  },
                  header: [
                    {
                        name: `${name}_tag`,
                        component: 'ZdTag',
                        tag: 'div',
                        cssStyle: 'display: flex; flex-direction: row; justify-content: space-between; align-items: center',
                        children: [
                            {
                                name: `${name}_title`,
                                component: 'ZdText',
                                cssStyle: 'color: #2d68c3;',
                                text: `${value.DSTITULOINF} (${i + 1}/${values.length})`,
                            },
                        ],
                    },
                  ],
                  children: [
                    contentContainer,
                    {
                        isVisible: false,
                        name: `${name}_id`,
                        component: 'ZdText',
                        text: value.NRINFECOMMERCE,
                    },
                  ],
                };
                items.push(item);
            }
            const list = this.modalNotificationList.children[1];
            if (list
                && this.modalNotificationList
                && this.modalNotificationList.children.length
                && this.modalNotificationList.children[0].children?.length
                && this.modalNotificationList.children[0].children[1].children?.length) {
                this.modalNotificationList.children[0].children[1].children[0].text += ` (${values.length})`;
                list.items = items;
                this.modalNotificationList.show();
            }
        }
    }

    private async notificationMarkAsRead(notificationId: any) {
        const uri = '/readNotification/save';
        const params = {
            NRINFECOMOPER: notificationId,
        };
        await Http.post(uri, params);
    }
}
